//////////////////////////////////////////////////////////////////////////////
// vballs.c                                                                 //
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// includes                                                                 //
//////////////////////////////////////////////////////////////////////////////
#include <gb.h>
#include "vballs.h"
#include "math.h"
#include "sincos.h"
#include "cube.h"
#include "scroller.h"

//////////////////////////////////////////////////////////////////////////////
// defines                                                                  //
//////////////////////////////////////////////////////////////////////////////
#define Sar(a,b) (((a)<0) ? -(-(a)>>(b)) : ((a)>>(b))) // shift ar. right
#define dist 220 //256

//////////////////////////////////////////////////////////////////////////////
// variables                                                                //
//////////////////////////////////////////////////////////////////////////////
unsigned int sprites[VERTICES][2]; // x,y
int ot[8][32];
int otn[8];		// length of ordering table entries

//////////////////////////////////////////////////////////////////////////////
// SetRotation                                                              //
//////////////////////////////////////////////////////////////////////////////
void SetRotation(unsigned int rx, unsigned int ry, unsigned int rz)
{	
	int s1,s2,s3,c1,c2,c3;
	int s2s3, c1c3, s1c3;

	c3 = ctab[rx]; c1 = ctab[ry]; c2 = ctab[rz];
	s3 = stab[rx]; s1 = stab[ry]; s2 = stab[rz];

	s2s3 = Mul(s2,s3);
	c1c3 = Mul(c1,c3);
	s1c3 = Mul(s1,c3);

	xx = c1c3 - Mul(s1,s2s3);
	xy = Mul(c2,s3);
	xz = s1c3 + Mul(c1,s2s3);
	
	yx = -Mul(c1,s3) - Mul(s1c3,s2);
	yy = Mul(c2,c3);
	yz = Mul(c1c3,s2) - Mul(s1,s3);
	
	zx = -Mul(s1,c2);
	zy = -s2;
	zz = Mul(c1,c2);
}

//////////////////////////////////////////////////////////////////////////////
// ClearOT                                                                  //
//////////////////////////////////////////////////////////////////////////////
void ClearOT()
{
	int i;
	for (i=0; i<8; i++)
	{
		otn[i] = 0;
	}
}

//////////////////////////////////////////////////////////////////////////////
// RotateAndProject                                                         //
//////////////////////////////////////////////////////////////////////////////
void RotateAndProject()
{
	unsigned int v;
	unsigned int *s = sprites[0];
	for (v=0; v<VERTICES; v++)
	{
		long zzz;
		x1 = vertices[v][0];
		y1 = vertices[v][1];
		z1 = vertices[v][2];

		RotateVertex();
		
		zzz = z2 + dist;

		z2 = ((long)z2+32) >> 3; // / (64/8);
		if (z2 < 1) z2 = 1;
		if (z2 > 7) z2 = 7;
		ot[z2][otn[z2]++] = v;

		*s++ = 80 + 4 + Sar((zzz * x2), 8);
		*s++ = 64 + 12 + Sar((zzz * y2), 8);
	}
}

//////////////////////////////////////////////////////////////////////////////
// DrawSprites                                                              //
//////////////////////////////////////////////////////////////////////////////
void DrawSprites()
{
	int i;
	unsigned int j, s=0;
	for (i=7; i>=0; i--)
	{
		for (j=0; j<otn[i]; j++)
		{
			unsigned int *p = sprites[ot[i][j]];
			move_sprite(s, p[0],p[1]);
			set_sprite_prop(s++, i);
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// InitVectorBalls                                                          //
//////////////////////////////////////////////////////////////////////////////
void InitVectorBalls()
{
	unsigned int s=0;
	InitSprites();
	for (s=0; s<VERTICES; s++)
	{
		set_sprite_tile(s, 1*2);
	}
	
	set_interrupts(VBL_IFLAG | LCD_IFLAG);
	STAT_REG = 0x40;
	LYC_REG = 0;

	OCPS_REG = 2*3 + 1;
}

//////////////////////////////////////////////////////////////////////////////
// VectorBalls                                                              //
//////////////////////////////////////////////////////////////////////////////
void VectorBalls()
{
	unsigned int rx=32, ry=128, rz=8;
	while (1)
	{
		if ((joypad() & (J_START+J_SELECT+J_A+J_B)) == (J_START+J_SELECT+J_A+J_B)) break;
		
		DoScroller();
		
		SetRotation(rx+=1, ry+=3, rz+=2);
		ClearOT();
		RotateAndProject();
		
		//while (LY_REG != 143);
		DrawSprites();
	}
}
